"""button_events module for EWOK.

Copyright (C) 2022 Theodore B. Ruegsegger
See LICENSE file for terms of use.
"""

# from global_constants import *
from design import *

def add_centerpoint(design, canvas):
    # Set mode to add a centerpoint (fixed pivot).
    if len(design.positions) == 0:
        warn_no_coupler_points(canvas)
        design.mode = "Init"
    elif design.is_design_complete():
        warn_design_full(canvas)
        design.mode = "Init"
    elif len(design.centerpoints) > len(design.circlepoints):
        # Don't make a 2nd M until the 1st has a K
        warn_complete_first_link(canvas)
        design.mode = "Init"
    else:
        notify(canvas, "Specifying centerpoint (fixed pivot).")
        design.mode = "AddCtrPt"
        ## else Popup explaining why
    canvas.configure(cursor=DEFAULT_CURSOR)

def add_circlepoint(design, canvas):
    # Set mode to add a circlepoint (moving pivot).
    if len(design.positions) == 0:
        warn_no_coupler_points(canvas)
        design.mode = "Init"
    elif design.is_design_complete():
        warn_design_full(canvas)
        design.mode = "Init"
    elif len(design.circlepoints) > len(design.centerpoints):
        # Don't make a 2nd K until the 1st has an M
        warn_complete_first_link(canvas)
        design.mode = "Init"
    else:
        notify(canvas, "Specifying circlepoint (moving pivot).")
        design.mode = "AddCirPt"
    canvas.configure(cursor=DEFAULT_CURSOR)

def add_position(design, canvas):
    # Set mode  to add a coupler position specification.
    if len(design.positions) > 0:
        design.redraw_last_figure(canvas)
    if len(design.positions) < 5:
        ## Test if other stuff exists; if so, resolve
        design.mode = "AddPosn"
    else:
        warn(canvas, "Can't specify more than 5 positions.")
        ## But suggest using display of incremental positions
        ## to solve approximately
        design.mode = "Select"
    canvas.configure(cursor=DEFAULT_CURSOR)

def add_sliderpoint(design, canvas):
    # Set mode to add a sliderpoint
    # (moving pivot with centerpoint at infinity).
    if len(design.positions) == 0:
        warn_no_coupler_points(canvas)
        design.mode = "Init"
    elif design.is_design_complete():
        warn_design_full(canvas)
        design.mode = "Init"
    elif len(design.circlepoints) > len(design.centerpoints):
        warn(canvas, "Add centerpoint to unpaired circlepoint first.")
    elif len(design.circlepoints) < len(design.centerpoints):
        warn(canvas, "Add centerpoint to unpaired centerpoint first.")
    else:
        design.mode = "AddSlider"
    canvas.configure(cursor=DEFAULT_CURSOR)

def add_turning_block(design, canvas):
    # Set mode to add a turning block (concurrency point;
    # fixed pivot with circlepoint at infinity).
    if len(design.positions) == 0:
        warn_no_coupler_points(canvas)
        design.mode = "Init"
    elif design.is_design_complete():
        warn_design_full(canvas)
        design.mode = "Init"
    elif len(design.circlepoints) > len(design.centerpoints):
        warn(canvas, "Add centerpoint to unpaired circlepoint first.")
    elif len(design.circlepoints) < len(design.centerpoints):
        warn(canvas, "Add centerpoint to unpaired centerpoint first.")
    else:
        t = "Beware! Turning blocks (concurrency points) not fully"
        t += " implemented."
        warn(canvas, t)
        design.mode = "AddTurnBlk"
    canvas.configure(cursor=DEFAULT_CURSOR)

def delete_object(design, canvas):
    # User wants to delete an object. Sets mode and cursor
    if (len(design.positions) + len(design.circlepoints)
        + len(design.centerpoints)) > 0:
        design.mode = "DelFig"
        canvas.configure(cursor=DELETE_CURSOR)
    else:
        canvas.configure(cursor=DEFAULT_CURSOR)
        design.mode = "Init"

def hide_coupler_frame(design, canvas):
    if len(design.positions) > 0:
        design.coupler_border.erase(canvas)

def notify(canvas, text):
    canvas.statusbar.update(text=text,
                            background=STATUS_DEFAULT_COLOR)

def show_coupler_frame(design, canvas):
    design.create_coupler_frame()
    design.draw_coupler_images(
        canvas, design.coupler_border, DESIGN_POSITION_COLOR)

def warn(canvas, text):
    canvas.statusbar.update(text=text, background=BG_HIGHLIGHT_COLOR)

def warn_complete_first_link(canvas):
    warn(canvas, "Complete the first link before starting a new one.")

def warn_design_full(canvas):
    warn(canvas, "Linkage is complete, can't add more points.")

def warn_no_coupler_points(canvas):
    warn(canvas, "Specify one or more coupler positions first.")

